import React, { useState, useEffect } from 'react';
import { Container, Typography, TextField, Button, Grid, Chip } from '@mui/material';
import axios from 'axios';

export default function EventForm({ initialValues, onSubmit }) {

  const formatDateTimeLocal = (dateTimeString) => {
    if (!dateTimeString) return '';
    const date = new Date(dateTimeString);
    return date.toISOString().slice(0, 16);
  };
  
  const formatDateTimeUTC = (dateTimeString) => {
    const date = new Date(dateTimeString);
    const offset = date.getTimezoneOffset(); // Minuten Offset zur UTC
    const adjustedDate = new Date(date.getTime() - (offset * 60 * 1000));
    return adjustedDate.toISOString();
  };

  const [formData, setFormData] = useState(initialValues || {
    name: '',
    location: '',
    website: '',
    description: '',
    startTime: '',
    endTime: '',
    groups: {
      roundRace: [],
      timeRace: [],
      startEndRace: []
    }
  });

  const [groups, setGroups] = useState([]);

  useEffect(() => {
    axios.get('/api/groups')
      .then(response => setGroups(response.data))
      .catch(error => console.error('Error loading groups:', error));
  }, []);

  useEffect(() => {
    if (initialValues) {
      setFormData({
        ...initialValues,
        startTime: formatDateTimeLocal(initialValues.startTime),
        endTime: formatDateTimeLocal(initialValues.endTime)
      });
    }
  }, [initialValues]);

  const handleChange = (e) => {
    const { name, value } = e.target;
    setFormData(prevData => ({
      ...prevData,
      [name]: value
    }));
  };

  const handleGroupToggle = (groupId, category) => {
    setFormData(prevData => {
      let updatedGroups = {
        roundRace: prevData.groups.roundRace.filter(g => g.groupId !== groupId),
        timeRace: prevData.groups.timeRace.filter(g => g.groupId !== groupId),
        startEndRace: prevData.groups.startEndRace.filter(g => g.groupId !== groupId)
      };

      if (!prevData.groups[category].some(g => g.groupId === groupId)) {
        updatedGroups[category].push({ groupId, value: '' });
      }

      return {
        ...prevData,
        groups: updatedGroups
      };
    });
  };

  const handleValueChange = (groupId, category, value) => {
    setFormData(prevData => {
      let updatedGroups = prevData.groups[category].map(g => 
        g.groupId === groupId ? { ...g, value } : g
      );

      return {
        ...prevData,
        groups: {
          ...prevData.groups,
          [category]: updatedGroups
        }
      };
    });
  };

  const handleSubmit = (e) => {
    e.preventDefault();
    const formattedData = {
      ...formData,
      startTime: formatDateTimeUTC(formData.startTime),
      endTime: formatDateTimeUTC(formData.endTime),
      groups: [
        ...formData.groups.roundRace.map(g => ({ groupId: g.groupId, raceType: 1, value: parseInt(g.value) || 0 })),
        ...formData.groups.timeRace.map(g => ({ groupId: g.groupId, raceType: 2, value: parseInt(g.value) || 0 })),
        ...formData.groups.startEndRace.map(g => ({ groupId: g.groupId, raceType: 3, value: parseInt(g.value) || 0 }))
      ]
    };
    onSubmit(formattedData);
  };

  return (
    <Container>
      <Typography variant="h4" component="h4" gutterBottom>
        {initialValues ? `Veranstaltung bearbeiten` : 'Neue Veranstaltung'}
      </Typography>
      <form onSubmit={handleSubmit}>
        <Grid container spacing={2}>
          <Grid item xs={12}>
            <TextField
              name="name"
              label="Name"
              variant="outlined"
              fullWidth
              value={formData.name || ''}
              onChange={handleChange}
              required
            />
          </Grid>
          <Grid item xs={12}>
            <TextField
              name="location"
              label="Ort"
              variant="outlined"
              fullWidth
              value={formData.location || ''}
              onChange={handleChange}
            />
          </Grid>
          <Grid item xs={12}>
            <TextField
              name="website"
              label="Webseite"
              variant="outlined"
              fullWidth
              value={formData.website || ''}
              onChange={handleChange}
            />
          </Grid>
          <Grid item xs={12}>
            <TextField
              name="description"
              label="Beschreibung"
              variant="outlined"
              fullWidth
              multiline
              rows={4}
              value={formData.description || ''}
              onChange={handleChange}
            />
          </Grid>
          <Grid item xs={12}>
            <TextField
              name="startTime"
              label="Startzeit"
              type="datetime-local"
              variant="outlined"
              fullWidth
              value={ formData.startTime || ''}
              onChange={handleChange}
              InputLabelProps={{
                shrink: true,
              }}
              required
            />
          </Grid>
          <Grid item xs={12}>
            <TextField
              name="endTime"
              label="Endzeit"
              type="datetime-local"
              variant="outlined"
              fullWidth
              value={ formData.endTime || ''}
              onChange={handleChange}
              InputLabelProps={{
                shrink: true,
              }}
              required
            />
          </Grid>
          {/* Der Rest des Codes bleibt unverändert */}
          <Grid item xs={12}>
            <Typography variant="h6">Rundenrennen</Typography>
            <Grid container spacing={1}>
              {groups.map(group => {
                const isSelected = formData.groups.roundRace.some(g => g.groupId === group.groupId);
                return (
                  <Grid item key={group.groupId}>
                    <div style={{ display: 'flex', alignItems: 'center' }}>
                      <Chip
                        label={group.name}
                        clickable
                        color={isSelected ? 'primary' : 'default'}
                        onClick={() => handleGroupToggle(group.groupId, 'roundRace')}
                        style={{ marginRight: 8 }}
                      />
                      {isSelected && (
                        <TextField
                          value={formData.groups.roundRace.find(g => g.groupId === group.groupId)?.value || ''}
                          onChange={(e) => handleValueChange(group.groupId, 'roundRace', e.target.value)}
                          placeholder="Anzahl"
                          size="small"
                          variant="outlined"
                          style={{ width: '70px' }}
                        />
                      )}
                    </div>
                  </Grid>
                );
              })}
            </Grid>
          </Grid>
          <Grid item xs={12}>
            <Typography variant="h6">Zeitrennen</Typography>
            <Grid container spacing={1}>
              {groups.map(group => {
                const isSelected = formData.groups.timeRace.some(g => g.groupId === group.groupId);
                return (
                  <Grid item key={group.groupId}>
                    <div style={{ display: 'flex', alignItems: 'center' }}>
                      <Chip
                        label={group.name}
                        clickable
                        color={isSelected ? 'primary' : 'default'}
                        onClick={() => handleGroupToggle(group.groupId, 'timeRace')}
                        style={{ marginRight: 8 }}
                      />
                      {isSelected && (
                        <TextField
                          value={formData.groups.timeRace.find(g => g.groupId === group.groupId)?.value / 1000 || ''}
                          onChange={(e) => handleValueChange(group.groupId, 'timeRace', e.target.value * 1000 )}
                          placeholder="Secunden"
                          size="small"
                          variant="outlined"
                          style={{ width: '70px' }}
                        />
                      )}
                    </div>
                  </Grid>
                );
              })}
            </Grid>
          </Grid>
          <Grid item xs={12}>
            <Typography variant="h6">Start/Ziel Rennen</Typography>
            <Grid container spacing={1}>
              {groups.map(group => {
                const isSelected = formData.groups.startEndRace.some(g => g.groupId === group.groupId);
                return (
                  <Grid item key={group.groupId}>
                    <div style={{ display: 'flex', alignItems: 'center' }}>
                      <Chip
                        label={group.name}
                        clickable
                        color={isSelected ? 'primary' : 'default'}
                        onClick={() => handleGroupToggle(group.groupId, 'startEndRace')}
                        style={{ marginRight: 8 }}
                      />
                    </div>
                  </Grid>
                );
              })}
            </Grid>
          </Grid>
          <Grid item xs={12}>
            <Button type="submit" variant="contained" color="primary">
              Speichern
            </Button>
          </Grid>
        </Grid>
      </form>
    </Container>
  );
}
