import { useState, useEffect } from 'react';
import { useRouter } from 'next/router';
import { Container, Typography, Table, TableBody, TableCell, TableContainer, TableHead, TableRow, Paper, FormControl, InputLabel, Select, MenuItem, Collapse, IconButton, Box, CircularProgress } from '@mui/material';
import { ExpandLess, ExpandMore } from '@mui/icons-material';
import axios from 'axios';

const ResultsPage = () => {
  const router = useRouter();
  const { id } = router.query;
  const [results, setResults] = useState([]);
  const [event, setEvent] = useState(null);
  const [loading, setLoading] = useState(true);
  const [ageCategory, setAgeCategory] = useState(1);
  const [groups, setGroups] = useState([]);
  const [openRows, setOpenRows] = useState({});

  const formatMilliseconds = (milliseconds) => {
    const hours = Math.floor(milliseconds / 3600000);
    const minutes = Math.floor((milliseconds % 3600000) / 60000);
    const seconds = Math.floor((milliseconds % 60000) / 1000);
    const millis = Math.floor((milliseconds % 1000) / 10);
  
    return `${hours.toString().padStart(2, '0')}:${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')},${millis.toString().padStart(2, '0')}`;
  };

  const calculateRundenzeit = (rounds, index) => {
    if (index == 0) {
      return rounds[index].round_time;
    }
    return rounds[index].round_time - rounds[index - 1].round_time;
  };

  const formatDateTimeLocal = (dateTimeString) => {
    if (!dateTimeString) return '';
    const date = new Date(dateTimeString);
    return date.toISOString().slice(0, 16);
  };
  
  const fetchResults = async () => {
    if (id && ageCategory && event) {
      setLoading(true);
      try {
        const selectedGroup = groups.find(group => group.groupId == ageCategory);
        const response = await axios.post(`/api/events/${id}/results`, {
          eventId: id,
          startTime: formatDateTimeLocal(event.startTime),
          endTime: formatDateTimeLocal(event.endTime),
          raceType: selectedGroup.raceType,
          competition: ageCategory,
          value: selectedGroup.value
        });
        setResults(response.data);
        setLoading(false);
      } catch (error) {
        console.error('Error fetching results:', error);
        setLoading(false);
      }
    }
  };

  const fetchRoundDetails = async (startNumber) => {
    if (id && ageCategory && event) {
      setLoading(true);
      try {
        const selectedGroup = groups.find(group => group.groupId == ageCategory);
        const response = await axios.post(`/api/events/${id}/results/${startNumber}`, {
          eventId: id,
          startTime: formatDateTimeLocal(event.startTime),
          endTime: formatDateTimeLocal(event.endTime),
          raceType: selectedGroup.raceType,
          competition: ageCategory,
          value: selectedGroup.value,
          startNumber: startNumber
        });
        setLoading(false);
        return response.data;
      } catch (error) {
        console.error('Error fetching results:', error);
        setLoading(false);
      }
    }
  };

  const handleRowClick = async (startNumber) => {
    if (openRows[startNumber]) {
      setOpenRows(prevState => ({ ...prevState, [startNumber]: !prevState[startNumber] }));
    } else {
      const roundDetails = await fetchRoundDetails(startNumber);
      setOpenRows(prevState => ({ ...prevState, [startNumber]: { open: true, rounds: roundDetails } }));
    }
  };

  useEffect(() => {
    const fetchEventData = async () => {
      try {
        const response = await axios.get(`/api/events/${id}`);
        setEvent(response.data);
        setLoading(false);
      } catch (error) {
        console.error('Error fetching event data:', error);
        setLoading(false);
      }
    };

    if (id) {
      fetchEventData();
    }
  }, [id]);

  useEffect(() => {
    const fetchGroupData = async () => {
      try {
        const response = await axios.get(`/api/events/${id}/groupMn`);
        setGroups(response.data);
        setLoading(false);
      } catch (error) {
        console.error('Error fetching group data:', error);
        setLoading(false);
      }
    };

    if (id) {
      fetchGroupData();
    }
  }, [id]);

  useEffect(() => {
    fetchResults();
  }, [id, groups, ageCategory]);

  if (loading) {
    return (
      <Container>
        <Typography variant="h4" component="h4" gutterBottom>
        Ergebnisse {event ? event.name : `Veranstaltung ${id}`}
        </Typography>
        <CircularProgress />
      </Container>
    );
  }

  return (
    <Container>
      <Typography variant="h4" component="h4" gutterBottom>
        Ergebnisse {event ? event.name : `Veranstaltung ${id}`}
      </Typography>
      <FormControl fullWidth margin="normal">
        <InputLabel id="age-category-label">Altersklasse</InputLabel>
        <Select
          labelId="age-category-label"
          value={ageCategory}
          onChange={(e) => setAgeCategory(e.target.value)}
        >
          {groups.map((group) => (
            <MenuItem key={group.Group.groupId} value={group.Group.groupId}>
              {group.Group.name}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
      <TableContainer component={Paper}>
        <Table>
          <TableHead>
            <TableRow>
              <TableCell></TableCell>
              <TableCell>Platz</TableCell>
              <TableCell>Startnummer</TableCell>
              <TableCell>Vorname</TableCell>
              <TableCell>Nachname</TableCell>
              <TableCell>Zeit</TableCell>
              <TableCell>Runden</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {results.map((result, index) => (
              <>
                <TableRow key={index} onClick={() => handleRowClick(result.startNumber)} style={{ cursor: 'pointer' }}>
                  <TableCell>
                    <IconButton aria-label="expand row" size="small">
                      {openRows[result.startNumber]?.open ? <ExpandLess /> : <ExpandMore />}
                    </IconButton>
                  </TableCell>
                  <TableCell>{result.rank}</TableCell>
                  <TableCell>{result.startNumber}</TableCell>
                  <TableCell>{result.first_name}</TableCell>
                  <TableCell>{result.last_name}</TableCell>
                  <TableCell>{formatMilliseconds(result.time)}</TableCell>
                  <TableCell>{result.rounds}</TableCell>
                </TableRow>
                {openRows[result.startNumber]?.open && (
                  <TableRow>
                    <TableCell colSpan={7} style={{ paddingBottom: 0, paddingTop: 0 }}>
                      <Collapse in={openRows[result.startNumber].open} timeout="auto" unmountOnExit>
                        <Box margin={1}>
                          <Typography variant="h6" gutterBottom component="div">
                            Rundenprotokoll
                          </Typography>
                          <Table size="small" aria-label="purchases">
                            <TableHead>
                              <TableRow>
                                <TableCell>Runde</TableCell>
                                <TableCell>Rundenzeit</TableCell>
                                <TableCell>Zeit</TableCell>
                              </TableRow>
                            </TableHead>
                            <TableBody>
                              {openRows[result.startNumber].rounds.map((round, roundIndex) => (
                                <TableRow key={roundIndex}>
                                  <TableCell>{roundIndex + 1}</TableCell>
                                  <TableCell>{formatMilliseconds(calculateRundenzeit(openRows[result.startNumber].rounds, roundIndex))}</TableCell>
                                  <TableCell>{formatMilliseconds(round.round_time)}</TableCell>
                                </TableRow>
                              ))}
                            </TableBody>
                          </Table>
                        </Box>
                      </Collapse>
                    </TableCell>
                  </TableRow>
                )}
              </>
            ))}
          </TableBody>
        </Table>
      </TableContainer>
    </Container>
  );
};

export default ResultsPage;
