import serial
import threading
import queue
import time
import mysql.connector
import json
import os
from datetime import datetime, timedelta

terminate_flag = False

# Konfigurationsdaten
def load_config():
    script_dir = os.path.dirname(__file__)
    config_path = os.path.join(script_dir, 'config.json')

    with open(config_path, 'r') as f:
        config = json.load(f)
    return config['event_id'], config['timing_point'], config['decoder_id'], int(config['timedelta']), config['connection_value']

def parse_epc(epc):
    if len(epc) == 24:  # 96-bit EPC
        header = epc[:2]
        epc_manager = epc[2:9]
        object_class = epc[9:15]
        serial_number = epc[15:]
    elif len(epc) == 32:  # 128-bit EPC
        header = epc[:2]
        epc_manager = epc[2:14]
        object_class = epc[14:20]
        serial_number = epc[20:]
    elif len(epc) == 40:  # 160-bit EPC
        header = epc[:2]
        epc_manager = epc[2:19]
        object_class = epc[19:25]
        serial_number = epc[25:]
    elif len(epc) == 48:  # 192-bit EPC
        header = epc[:2]
        epc_manager = epc[2:24]
        object_class = epc[24:30]
        serial_number = epc[30:]
    else:
        return None

    return header, epc_manager, object_class, serial_number

def read_from_serial(serial_port, baud_rate, timeout, data_queue):
    ser = serial.Serial(serial_port, baud_rate, timeout=timeout)
    startTime = time.time()* 1000
    try:
        print("Reading from serial port...")
        while not terminate_flag:
            command = '\nU\r'
            ser.write(command.encode('utf-8'))
            while ser.in_waiting > 0:
                response = ser.readline().decode('utf-8', errors='ignore').strip()
                if response.startswith('U'):
                    epc_code = response[1:].strip()
                    data_queue.put((epc_code, (time.time()* 1000) - startTime, datetime.now()))
    except KeyboardInterrupt:
        print("Serial reading interrupted")
    finally:
        ser.close()

def process_data(data_queue, event_id, timing_point, decoder_id, delta):
    while not terminate_flag:
        try:
            db_conn = mysql.connector.connect(
                host="localhost",
                user="jan",
                password="admin",
                database="event_management"
            )
            cursor = db_conn.cursor()
            epc_hits = {}

            while not terminate_flag:
                if not data_queue.empty():
                    epc_code, timestamp, datetimestamp = data_queue.get()
                    epc_parts = parse_epc(epc_code)
                    if epc_parts:
                        # header, epc_manager, object_class, serial_number = epc_parts
                        if epc_code not in epc_hits:
                            epc_hits[epc_code] = {'hits': 1, 'timestamp': timestamp, 'datetimestamp': datetimestamp}
                        else:
                            epc_hits[epc_code]['hits'] += 1

                for epc_code, epc_hit in list(epc_hits.items()):
                    if datetime.now() - epc_hit['datetimestamp'] > timedelta(seconds=delta):
                        cursor.execute(
                            "INSERT INTO tracking (transponder, eventId, time, hits, timingPoint, decoderId, utcTime) VALUES (%s, %s, %s, %s, %s, %s, %s)",
                            (epc_code, event_id, epc_hit['timestamp'], epc_hit['hits'], timing_point, decoder_id, epc_hit['datetimestamp'])
                        )
                        db_conn.commit()
                        print(epc_code)
                        del epc_hits[epc_code]

        except mysql.connector.Error as err:
            print(f"Fehler: {err}")
            time.sleep(5)

        finally:
            if db_conn.is_connected():
                cursor.close()
                db_conn.close()

if __name__ == "__main__":
    # serial_port = "COM5"
    baud_rate = 38400
    timeout = 1
    data_queue = queue.Queue()

    # in der Config noch schreiben ob rundstrecke und delta
    event_id, timing_point, decoder_id, delta, serial_port = load_config()

    serial_thread = threading.Thread(target=read_from_serial, args=(serial_port, baud_rate, timeout, data_queue))
    process_thread = threading.Thread(target=process_data, args=(data_queue, event_id, timing_point, decoder_id, delta))

    serial_thread.start()
    process_thread.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        terminate_flag = True

    serial_thread.join()
    process_thread.join()